/******************************************************************************
 * Clause.h
 *
 * GDSAT
 *
 ******************************************************************************
 * Copyright (C) 2013 Benjamin Niedermann <niedermann@kit.edu>
 *
 * This program is free software: you can redistribute it and/or modify it
 * under the terms of the GNU General Public License as published by the Free
 * Software Foundation, either version 3 of the License, or (at your option)
 * any later version.
 *
 * This program is distributed in the hope that it will be useful, but WITHOUT
 * ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
 * FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License for
 * more details.
 *
 * You should have received a copy of the GNU General Public License along with
 * this program.  If not, see <http://www.gnu.org/licenses/>.
 *****************************************************************************/

#ifndef CNF_H_
#define CNF_H_
#include <vector>
#include <sstream>
#include <sys/wait.h>
#include <stdio.h>
#include <sys/types.h>
#include <fcntl.h>
#include <stdlib.h>
#include <limits.h>
#include <iostream>
#include <stdexcept>
using namespace std;

/**
 * Represents a clause in CNF.
 */
class Clause {
private:

	int * variables;
	bool * sign;
	int size;
	int pos;
public:
	Clause();
	Clause(int size);
	virtual ~Clause();


	/**
	 * Reserves storage for a given number of
	 * literals. It is essential that this method
	 * is called before variables are added to
	 * this clause.
	 */
	void reserve(int literalCount){
		variables = new int[literalCount];
		sign      = new bool[literalCount];
		pos =0;
		size =literalCount;
	}

	/**
	 * Adds a variable to the clause.
	 * -positive: true if the literal is positive otherwise false.
	 */
	void add(int  var, bool positive){

		variables[pos]=var;
		sign[pos] = positive;
		pos++;
	}

	int getSize(){
		return size;
	}

	/**
	 * Adds the literals of a given clause to
	 * this clause.
	 */
	void add(Clause & cnf){
		for(int i=0; i < cnf.size; ++i){
			add(cnf.variables[i],cnf.sign[i]);
		}
	}

	bool isEmpty(){
		return size == 0;
	}

	void write(ostream & s){
		for( int i=0; i <size; ++i){
			int var = variables[i];
			var++;
			var = sign[i] ? var : (-1*var);
			s <<  var << " ";
		}
		s << "0\n";
	}

	void write(FILE * file){
		stringstream  s;
		for( int i=0; i < size; ++i){
			int var = variables[i];
			var++;
			var = sign[i] ? var : (-1*var);
			s <<  var << " ";
		}
		string temp = s.str();
		temp +="0 ";
		fputs(temp.c_str(),file);
	}

	string toString(){
		stringstream s;
		write(s);
		return s.str();
	}

	void fill(bool * vars , int max){
		for(int i=0; i < size; ++i){
			if(max <= variables[i]){
				cout <<  variables[i] <<" exceeds varCount";
				throw "variable exceeds varCount";
			}
			vars[variables[i]] = true;
		}
	}
};

#endif /* CNF_H_ */
