/******************************************************************************
 * Clock.cpp
 *
 * GDSAT
 *
 ******************************************************************************
 * Copyright (C) 2013 Benjamin Niedermann <niedermann@kit.edu>
 *
 * This program is free software: you can redistribute it and/or modify it
 * under the terms of the GNU General Public License as published by the Free
 * Software Foundation, either version 3 of the License, or (at your option)
 * any later version.
 *
 * This program is distributed in the hope that it will be useful, but WITHOUT
 * ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
 * FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License for
 * more details.
 *
 * You should have received a copy of the GNU General Public License along with
 * this program.  If not, see <http://www.gnu.org/licenses/>.
 *****************************************************************************/

#ifndef CLOCK_H_
#define CLOCK_H_
#include <list>
#include <sys/time.h>
#include <iostream>
#include <iomanip>

using namespace std;
/**
 * Helper class that wraps a mechanism for measuring time.
 */
class Clock {
private:
	double overallTime;
    timeval startTime;
    list<double> times;
    bool startNecessary;
public:
	Clock();
	virtual ~Clock();

	void start(){
		  gettimeofday(&startTime, 0);
		  startNecessary = false;
	}

	/**
	 * Measures the time elapsed since calling method start and stores this time.
	 */
	double stop(){
		if(startNecessary){throw "start must be called before";}
		startNecessary = true;
		timeval end;
		gettimeofday(&end, 0);
		double time= ((double)end.tv_sec+(double)end.tv_usec/1000000.0)-((double)startTime.tv_sec+(double)startTime.tv_usec/1000000.0);
		if(time < 0.0001){time =0;}
		times.push_back(time);
		overallTime += time;
		return time;

	}

	double getOverallTime() const{
		return overallTime;
	}

	void write(ostream & stream) {
		for(list<double>::iterator it= times.begin(); it != times.end(); ++it){

			stream << " " << *it;
		}
	}


};

#endif /* CLOCK_H_ */
