/******************************************************************************
 * Model.h
 *
 * GDSAT
 *
 ******************************************************************************
 * Copyright (C) 2013 Benjamin Niedermann <niedermann@kit.edu>
 *
 * This program is free software: you can redistribute it and/or modify it
 * under the terms of the GNU General Public License as published by the Free
 * Software Foundation, either version 3 of the License, or (at your option)
 * any later version.
 *
 * This program is distributed in the hope that it will be useful, but WITHOUT
 * ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
 * FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License for
 * more details.
 *
 * You should have received a copy of the GNU General Public License along with
 * this program.  If not, see <http://www.gnu.org/licenses/>.
 *****************************************************************************/

#ifndef MODEL_H_
#define MODEL_H_
#include "./../common/Clause.h"




/**
 * Represents a range of variables.
 */
struct Range{
	int begin; // the index of the first variable.
	int end; // the index+1 of the last variable.

	Range(int begin, int end):begin(begin),end(end){}
	Range(){};
	int getVar(int pos){return begin+pos;}
};

class CNFModel {

private:


	Clause * cnfs; // clauses in CNF stored by this model
	/**
	 * pointer on the next clause that has not been initialized yet.
	 */
	Clause * nextCNF;

	/**
	 * Next index of variable that has not yet been used.
	 */
	int nextVar;


	/**
	 * in case that this model has already been solved this array contains
	 * the values of the variables.
	 */
	bool * values;

	/**
	 * Upper bound on the number of clauses used by this model
	 */
	int cnfCount;
	/**
	 * Upper bound on the number of variables used by this model
	 */
	int varCount;

	/**
	 * Number of clauses that is actually used, i.e., has been initialized by
	 * the method next.
	 */
	int usedCNF;

	int temp;

public:
	CNFModel();
	CNFModel(int varCount, int cnfCount);
	virtual ~CNFModel();

	/**
	 * Returns the value of the variable with index var.
	 * This method may only be used when the model has already
	 * been solved by a SAT solver.
	 */
	bool getValue(int var){return values[var];}

	void init(int varCount, int cnfCount){
		values    = new bool[varCount];
		cnfs      = new Clause[cnfCount];
		nextCNF   = 0;
		nextVar   = 0;
		usedCNF   =0;
		this->varCount = varCount;
		this->cnfCount = cnfCount;
	}

	Range reserveVariables(int number){

		int begin = nextVar;
		nextVar += number;
	//	temp+= number;
		return Range(begin,nextVar);
	}

	/**
	 * Call this method in order to gain the next clause to be
	 * defined.
	 * - literalCount: number of literals that belong to that clause.
	 */
	Clause * next(int literalCount){
		if(temp >= cnfCount ){
			cout << "no further cnfs possible" << endl;
			throw "no further cnfs possible";
		}

		if(nextCNF == 0){
			nextCNF = cnfs;
		}else{
			nextCNF++;
		}
		usedCNF++;
		temp++;

		nextCNF->reserve(literalCount);
		return nextCNF;
	}

	int reserved(){
		return temp;
	}



	void write(ostream & output);
	bool interpret(istream & stream);
	string write(FILE & file);

};

 /* namespace sat1d */
#endif /* MODEL_H_ */
