/******************************************************************************
 * Pathwidth.h
 *
 * GDSAT
 *
 ******************************************************************************
 * Copyright (C) 2013 Benjamin Niedermann <niedermann@kit.edu>
 *
 * This program is free software: you can redistribute it and/or modify it
 * under the terms of the GNU General Public License as published by the Free
 * Software Foundation, either version 3 of the License, or (at your option)
 * any later version.
 *
 * This program is distributed in the hope that it will be useful, but WITHOUT
 * ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
 * FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License for
 * more details.
 *
 * You should have received a copy of the GNU General Public License along with
 * this program.  If not, see <http://www.gnu.org/licenses/>.
 *****************************************************************************/


#ifndef PATHWIDTH_H_
#define PATHWIDTH_H_

#include <sstream>

#include "./../common/Graph.h"
#include "../common/Clock.h"
#include "./../common/ProblemSolver.h"
#include "./../common/Box.h"



class PWModel;
typedef pair<int,int> NodeInterval;

using namespace std;

/**
 * Creates a CNF model that is satisfied if and only if the parameter pathwidth
 * is an upper bound for the pathwidth of the graph.
 */
PWModel * createPWModel(int pathwidth, Graph & graph, Clock& clock);



/**
 * Stores an CNF model and its corresponding interpretation.
 * The array nodes contains for each node the range of variables.
 */
class PWModel : public ProblemModel{
	void print(Graph & graph);
public:
	int 	pathwidth;
	int length;
	Box * nodes;
    Range * edges;

	PWModel(int pathwidth,int varCount, int cnfCount) : ProblemModel(varCount,cnfCount) , pathwidth(pathwidth){
		nodes = 0;
		edges =0;
	}

	virtual ~PWModel(){
		if(nodes!= 0)delete [] nodes;
		if(edges!= 0)delete [] edges;
	}

	virtual void checkForFeasibility(Graph & graph);

	virtual int getValue(){
		return pathwidth;
	}

};



class PWSolver : public ProblemSolver{

public:


	virtual ProblemModel * createModel(Graph & graph,Clock & clock, int turn, Config * config){
		PWModel * model = createPWModel(turn+1,graph,clock);
		model->pathwidth = turn+1;
		return model;
	}

	virtual bool  analyze(SolverResult & result, ProblemModel * model, Result & solution){
		return result == SAT;
	}

	virtual bool  nextTurn(Graph & graph, int currentTurn, SolverResult & result){
		return currentTurn <= graph.n();

	}
};


#endif /* PATHWIDTH_H_ */
