/******************************************************************************
 * STOrientation.h
 *
 * GDSAT
 *
 ******************************************************************************
 * Copyright (C) 2013 Benjamin Niedermann <niedermann@kit.edu>
 *
 * This program is free software: you can redistribute it and/or modify it
 * under the terms of the GNU General Public License as published by the Free
 * Software Foundation, either version 3 of the License, or (at your option)
 * any later version.
 *
 * This program is distributed in the hope that it will be useful, but WITHOUT
 * ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
 * FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License for
 * more details.
 *
 * You should have received a copy of the GNU General Public License along with
 * this program.  If not, see <http://www.gnu.org/licenses/>.
 *****************************************************************************/


#ifndef ORIENTATIONSOLVER_H_
#define ORIENTATIONSOLVER_H_

#include "./../common/Graph.h"
#include "./../common/Clock.h"
#include "./../common/SATSolver.h"
#include "./../common/ProblemSolver.h"
#include "./../common/Box.h"





class STOModel;

typedef pair<int,int> EdgeInterval;

/**
 * Creates a CNF model that is satisfied if and only if the parameter k
 * is an upper bound for the height of the st-orientation.
 */

STOModel * createSTOModel(int s, int t, int k, Graph & graph, Clock& clock);



/**
 * Stores an CNF model and its corresponding interpretation.
 * The array nodes contains for each node the range of variables.
 */
class STOModel : public ProblemModel{
private:
	int getNodePosition(int node);
	void print(Graph & graph);

public:
	int     s;
	int     t;
	int 	stheight;
	int length;
	Range * nodes;
    Box * edges;

	STOModel(int s, int t, int k,int varCount, int cnfCount) : ProblemModel(varCount,cnfCount) , s(s),t(t),stheight(k),length(k+2){
		nodes = 0;
		edges =0;
	}

	virtual ~STOModel(){
		if(nodes!= 0)delete [] nodes;
		if(edges!= 0)delete [] edges;
	}

	virtual void checkForFeasibility(Graph & graph);

	virtual int getValue(){
		return stheight;
	}
};


struct STOConfig : public Config{
	int s;
	int t;
	STOConfig(int s, int t) : s(s), t(t){

	}
};

class STOrientationSolver : public ProblemSolver{

public:

	virtual ProblemModel * createModel(Graph & graph,Clock & clock, int turn, Config * config){
		STOConfig * stoc = (STOConfig*)config;
		STOModel * model = createSTOModel(stoc->s, stoc->t,turn+1,graph,clock);
		model->stheight = turn+1;
		return model;
	}

	virtual bool  analyze(SolverResult & result, ProblemModel * model, Result & solution){
		return result == SAT;
	}

	virtual bool  nextTurn(Graph & graph, int currentTurn, SolverResult & result){
		return currentTurn <= graph.n();

	}
};

#endif /* ORDERSOLVER_H_ */
