/******************************************************************************
 * Boxicity.cpp
 *
 * GDSAT
 *
 ******************************************************************************
 * Copyright (C) 2013 Benjamin Niedermann <niedermann@kit.edu>
 *
 * This program is free software: you can redistribute it and/or modify it
 * under the terms of the GNU General Public License as published by the Free
 * Software Foundation, either version 3 of the License, or (at your option)
 * any later version.
 *
 * This program is distributed in the hope that it will be useful, but WITHOUT
 * ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
 * FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License for
 * more details.
 *
 * You should have received a copy of the GNU General Public License along with
 * this program.  If not, see <http://www.gnu.org/licenses/>.
 *****************************************************************************/
#include <utility>
#include <set>
#include "Boxicity.h"
#include "Grid.h"
#include "../common/Clause.h"
#include "../common/Box.h"

using namespace std;




BCModel * createBCModel(int width, int height, Graph & graph, Clock& clock){

	clock.start();

	Box * nodes = new Box[graph.n()];
	int *sizes = new int[2];
	width = width+2;
	height = height+2;
	sizes[0] = width;
	sizes[1] = height;

	int length = width * height;


	Grid *       edges = new Grid[graph.m()];


	int varCount = Box::varCount(2,sizes)*graph.n()+length* graph.m();
	int cnfCount = graph.m()
			       + 2* graph.m()*width*height
			       + graph.n()*(graph.n()-1)/2 *width*height
			       +Box::cnfCount(2,sizes)*graph.n();



	BCModel * bcModel = new BCModel(varCount,cnfCount);

	CNFModel & model = bcModel->getCNFModel();

	bcModel->nodes = nodes;
	bcModel->width = width;
	bcModel->height = height;


	for(int i=0; i < graph.n(); ++i){
		nodes[i].init(model,sizes,2);
	}


    for(int i=0; i < graph.m(); ++i){
		edges[i].init(width,height,model);
	}





    for(int i=0; i < graph.m(); ++i){
    	pair<int,int> edge = graph.edges()[i];
    	int x = edges[i].get();
    	int x1 = nodes[edge.first].get();
    	int x2 = nodes[edge.second].get();
    	Clause * cnf1 = model.next(length);
    	for(int pos =0; pos < length; ++pos,++x,++x1,++x2){
    		cnf1->add(x,true);
    		Clause * cnf2 = model.next(2);
    		cnf2->add(x,false);
    		cnf2->add(x1,true);
    		cnf2 = model.next(2);
    		cnf2->add(x,false);
    		cnf2->add(x2,true);
    	}
    }



    for(int i=0; i < graph.n(); ++i){
    	for(int j=i+1; j < graph.n(); ++j){

    		if(!graph.areAdjacent(i,j) && i!=j){

    			int x1 = nodes[i].get();
    			int x2 = nodes[j].get();
    			for(int pos=0; pos < length; ++pos,++x1,++x2){
    				Clause * cnf = model.next(2);

    	    		cnf->add(x1,false);
    	    		cnf->add(x2,false);

    			}
    		}
    	}

    }




	clock.stop();


	return bcModel;

}


void BCModel::print(Graph & graph){
	CNFModel & model = getCNFModel();
	for(int i=0; i < graph.n(); i++){
		cout << "Node " << i << endl;
		cout << nodes[i].toString(model) << endl;
	}

	string * canvas = new string[width*height];


	cout << "WRITE MODEL " << endl;


	for(int row =0; row < height; ++row){
		for(int col =0; col < width; ++col){
			cout << canvas[row*width+col] << "\t";
		}
		cout << endl;
	}



}

void BCModel::checkForFeasibility(Graph & graph){

	print(graph);

	int * pos = new int[graph.n()];
	for(int i=0; i < graph.n(); ++i){
		pos[i] = nodes[i].get();
	}

	set<int> identifiedEdges;

	for(int k=0; k < width*height; ++k){

		list<int> placedNodes;

		for(int i=0; i < graph.n(); ++i){
			if(getCNFModel().getValue(pos[i])){
				placedNodes.push_back(i);
			}
			++pos[i];
		}

		for(list<int>::iterator it = placedNodes.begin(); it != placedNodes.end(); ++it){
			list<int>::iterator it2 = list<int>::iterator(it);
			++it2;
			for(; it2 != placedNodes.end(); ++it2){
				int edge =graph.getEdge(*it,*it2);
				if(edge == -1){
					cout << *it << " " << *it2 << " are not adjacent";
					throw "Nodes are not adjacent!";
				}
				identifiedEdges.insert(edge);
			}
		}
	}

	if(identifiedEdges.size() != (unsigned int)graph.m()){
		cout << "Not all edges have been found!" << endl;
		throw "Not all edges have been found!" ;
	}
	int i=0;
	for(set<int>::iterator it = identifiedEdges.begin(); it != identifiedEdges.end(); ++it,++i){
		if(*it != i){
			cout << "wrong edge has been encountered.";
			throw"wrong edge has been encountered.";
		}
	}

	cout << "Test has been successfully executed!";

}


