/******************************************************************************
 * Grid.h
 *
 * GDSAT
 *
 ******************************************************************************
 * Copyright (C) 2013 Benjamin Niedermann <niedermann@kit.edu>
 *
 * This program is free software: you can redistribute it and/or modify it
 * under the terms of the GNU General Public License as published by the Free
 * Software Foundation, either version 3 of the License, or (at your option)
 * any later version.
 *
 * This program is distributed in the hope that it will be useful, but WITHOUT
 * ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
 * FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License for
 * more details.
 *
 * You should have received a copy of the GNU General Public License along with
 * this program.  If not, see <http://www.gnu.org/licenses/>.
 *****************************************************************************/

#ifndef GRID_H_
#define GRID_H_


#include <vector>

#include "./../common/Clause.h"
#include "./../common/Model.h"

using namespace std;

/**
 * Represents a grid consisting of variables.
 */
class Grid {

private:
    Range range;
    int width;
    int height;



public:

	Grid(){}
	virtual ~Grid();

	int get(int row, int col){
		return  range.begin+row * width +col;
	}

	int get(int row){
		return range.begin+row*width;
	}

	int get(){
		return range.begin;
	}

	void init(int width, int height, CNFModel & model);






	int rows() { return height;}
	int cols() { return width;}

    int getWidth(){
    	return width;
    }

    int getHeight(){
    	return height;
    }

    int getLength(){
    	return width*height;
    }

    string  toString(CNFModel model){
    	string res = "";
    	int  x = range.begin;
		for(int row =0; row < height; ++row){

     	    for(int col =0; col < width; ++col,++x){
     	    	if(model.getValue(x)){
     	    		res+= "1 ";
     	    	}else{
     	    		res += "0 " ;
     	    	}

		    }
     	    res+="\n";

		}
		return res;
    }

};


#endif /* GRID_H_ */
