/******************************************************************************
 * VisibilityRepresentation.h
 *
 * GDSAT
 *
 ******************************************************************************
 * Copyright (C) 2013 Benjamin Niedermann <niedermann@kit.edu>
 *
 * This program is free software: you can redistribute it and/or modify it
 * under the terms of the GNU General Public License as published by the Free
 * Software Foundation, either version 3 of the License, or (at your option)
 * any later version.
 *
 * This program is distributed in the hope that it will be useful, but WITHOUT
 * ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
 * FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License for
 * more details.
 *
 * You should have received a copy of the GNU General Public License along with
 * this program.  If not, see <http://www.gnu.org/licenses/>.
 *****************************************************************************/

#ifndef VISREP_H_
#define VISREP_H_


#include "./../common/Graph.h"
#include "./../common/Clock.h"
#include "./../common/SATSolver.h"
#include "./../common/ProblemSolver.h"
#include "./../common/Box.h"
#include "Grid.h"

class VRModel;
/**
 * Creates a CNF model that is satisfied if and only if the visibility representation
 * of the given graph can be drawn on a grid of size width x height.
 */
VRModel * createVRModel(int width, int height,bool oneBar, Graph & graph, Clock& clock);


class VRModel : public ProblemModel{
	void print(Graph & graph);
public:

	Box   * nodes;
	Box   * edges;
	Grid  * neighbors;
	int     width;
	int 	height;
    bool    oneBar;

	VRModel(int varCount, int cnfCount) : ProblemModel(varCount,cnfCount) {

	}

	virtual ~VRModel(){
		delete [] nodes;
		delete [] neighbors;
		delete [] edges;
	}

	virtual void checkForFeasibility(Graph & graph);

	virtual int getValue(){
		return width;
	}
};

struct VRConfig : public Config{
	bool oneBar;
	VRConfig(bool oneBar) : oneBar(oneBar) {};
};


class VRSolver : public ProblemSolver{
public:

	virtual ProblemModel * createModel(Graph & graph,Clock & clock, int turn, Config * config){
		VRConfig * vrc = (VRConfig*) config;
		return createVRModel(turn+1,graph.n()+2,vrc->oneBar,graph,clock);
	}

	virtual bool  analyze(SolverResult & result, ProblemModel * model, Result & solution){
		return result == SAT;
	}

	virtual bool  nextTurn(Graph & graph, int currentTurn, SolverResult & result){
		return currentTurn <= ((graph.n() > graph.n()*2-4) ? graph.n() : (graph.n()*2-4));

	}
};


#endif
