/******************************************************************************
 * utils.cpp
 *
 * GDSAT
 *
 ******************************************************************************
 * Copyright (C) 2013 Benjamin Niedermann <niedermann@kit.edu>
 *
 * This program is free software: you can redistribute it and/or modify it
 * under the terms of the GNU General Public License as published by the Free
 * Software Foundation, either version 3 of the License, or (at your option)
 * any later version.
 *
 * This program is distributed in the hope that it will be useful, but WITHOUT
 * ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
 * FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License for
 * more details.
 *
 * You should have received a copy of the GNU General Public License along with
 * this program.  If not, see <http://www.gnu.org/licenses/>.
 *****************************************************************************/


#include <sys/time.h>
#include <sstream>
#include <iostream>
#include <algorithm>
#include <list>
#include <fstream>
#include <iomanip>

#include "../common/Graph.h"

using namespace std;

void createGraph(Graph & graph,int argc, const char* argv[]){
	std::stringstream sn(argv[1]);
	//std::stringstream sm(argv[2]);
	int n;
	sn >> n;


	for(int i=0; i < n; ++i){
		graph.addNode(i);
	}
	for(int i=2; i < argc; i+=2){
		std::stringstream node1(argv[i]);
		std::stringstream node2(argv[i+1]);
		int id1, id2;
		node1 >> id1;
		node2 >> id2;
		graph.addEdge(id1,id2);
	}
}



void readFile(Graph & graph,list<pair<int,int> >& stNodes, istream & input){


	string buf;
	string line;
	while(std::getline(input,line)){
		std::stringstream stream(line);
		string word;
		if(!(stream >> word)){
			continue;
		}
		if(word.compare("nodeCount")==0){
			int nodeCount=0;
			stream >> nodeCount;
			for(int i=0; i < nodeCount; ++i){
				graph.addNode(i);
			}

		}
		if(word.compare("edges")==0){
			int id1;
			int id2;
			while(stream >> id1){
				stream >> id2;
				graph.addEdge(id1,id2);
			}
		}
		if(word.compare("st")==0){
			int id1;
			int id2;

			while(stream >> id1){
				stream >> id2;
	     		stNodes.push_back(pair<int,int>(id1,id2));
			}
		}
	}

}


void createGraph(Graph & graph, string & input){

	stringstream ss(input); // Insert the string into a stream

	string buf; // Have a buffer string

	vector<string> tokens; // Create vector to hold our words


	while (ss >> buf){
		std::stringstream par(buf);
	    tokens.push_back(buf);
	}

	std::stringstream par(tokens[0]);
	int n; par >> n;
	for(int i=0; i < n; ++i){
		graph.addNode(i);
	}

	for(unsigned int i=1; i < tokens.size(); i+=2){
		std::stringstream par1(tokens[i]);
		std::stringstream par2(tokens[i+1]);
		int id1, id2;
	    par1 >> id1;
	    par2 >> id2;
	    graph.addEdge(id1,id2);


	}

}


void normalize(Graph & graph, int s, int t){

	int * nodes = new int[graph.n()];

	for(int i=0; i < graph.n(); ++i){
		nodes[i]=0;
	}

	for(int i=0; i < graph.m(); ++i){
		nodes[graph.edges()[i].first]  += 1;
		nodes[graph.edges()[i].second] += 1;
	}

	for(int i=0; i < graph.n(); ++i){
		if(nodes[i]==1 && i!= s && i!= t){
			graph.addEdge(i,t);
		}
	}

	graph.addEdge(s,t);

	delete [] nodes;

}



